function tree = mrDMD(Xraw, dt,max_cyc, L)
% function tree = mrDMD(Xraw, dt, r, max_cyc, L)

% Inputs:
% Xraw      n by m matrix of raw data,
%           n measurements, m snapshots
% dt        time step of sampling
% r         rank of truncation
% max_cyc   to determine rho, the freq cutoff, compute
%           oscillations of max_cyc in the time window
% L         number of levels remaining in the recursion
% Xraw= X_inner107;
% T = size(Xraw, 2) * dt;
rho = max_cyc/T; % high freq cutoff at this level
sub = ceil(1/rho/8/pi/dt); % 4x Nyquist for rho  Բȡֵ

%% DMD at this level
Xaug = Xraw(:, 1:sub:end); % subsample
% Xaug = [Xaug(:, 1:end-1); Xaug(:, 2:end)];
XX = Xaug(:, 1:end-1);
Yp = Xaug(:, 2:end);
[U,Q,V] = svd([XX;Yp]);
Eig=diag(Q);
[J,K]=size(Eig);
energy(1)=0;
for i=1:J;
    energy(i+1)=energy(i)+Eig(i)^2;
end
throhold=0.999999*energy(J+1);
threhold=find(energy<throhold);
[J,k]=size(threhold);
V = V(:,1:k);

X = XX*V;
Yp = Yp*V;

[U, S, V] = svd([X]);
r = size(S,2);
U_r = U(:, 1:r); % rank truncation
S_r = S(1:r, 1:r);
V_r = V(:, 1:r);

Atilde = U_r' * Yp * V_r  * inv(S_r);
[W, D] = eig(Atilde);
lambda = diag(D);
Phi = Yp * V(:,1:r)* inv( S(1:r,1:r)) * W;
omega = log(lambda)/sub/2/pi/dt; % continuous-time eigenvalues
%% Compute DMD mode amplitudes b
b = Phi\ X;
%% DMD reconstruction
mm1 = size(X, 2); % mm1 = m - 1
time_dynamics = zeros(r, mm1);
t= (0:mm1-1)*dt; % time vector
for iter = 1:mm1,
     time_dynamics(:,iter) = (b*exp(omega*t(iter)));
end;

x_dmd= Phi * time_dynamics;

% % %% compute power of modes
% % Vand = zeros(r, size(X, 2)); % Vandermonde matrix
% % for k = 1:size(X, 2),
% %     Vand(:, k) = lambda.^(k-1);
% % end;
% % 
% % % the next 5 lines follow Jovanovic et al, 2014 code:
% % G = S_r * V_r';
% % P = (W'*W).*conj(Vand*Vand');
% % q = conj(diag(Vand*G'*W));
% % Pl = chol(P,'lower');
% % b = (Pl')\(Pl\q); % Optimal vector of amplitudes b

%% consolidate slow modes, where abs(omega) < rho
rho=0.5*mean(abs(omega));
mymodes_lowrank = find(abs(omega) <= rho);
omega_lowrank=find(omega(mymodes_lowrank));
mm1 = size(  X, 2); % mm1 = m - 1
time_dynamics_lowrank = zeros(numel(mymodes_lowrank), mm1);
for iter = 1:mm1,
    time_dynamics_lowrank(:,iter) = (b(mymodes_lowrank).*exp(omega(mymodes_lowrank)*t(iter)));
end;
 Phi_lowrank= Phi(:, mymodes_lowrank);
x_dmd_lowrank=  Phi_lowrank * time_dynamics_lowrank;


mymodes_sparse = find(abs(omega) > rho);
time_dynamics_sparse = zeros(numel(mymodes_sparse ), mm1);
omega_sparse=find(omega(mymodes_sparse));
for iter = 1:mm1,
    time_dynamics_sparse(:,iter) = (b(mymodes_sparse ).*exp(omega(mymodes_sparse )*t(iter)));
end;
Phi_sparse= Phi(:, mymodes_sparse);
x_dmd_sparse=Phi_sparse * time_dynamics_sparse  ;

thislevel.sub=sub; 
thislevel.X= X;  
thislevel.Yp= Yp;  
thislevel.r = r;   %%%ʱ
thislevel.omega = omega;  %%ȳɷ
thislevel.omega_lowrank = omega_lowrank;  %%ȳɷ
thislevel.omega_sparse = omega_sparse;  %%ȳɷ
thislevel.hit = numel(mymodes_lowrank) > 0;   %%%бmymodes0
thislevel.x_dmd= x_dmd;   
thislevel.x_dmd_lowrank = x_dmd_lowrank;   %%ģ̬Ӧķֵ
thislevel.x_dmd_sparse = x_dmd_sparse;  %%ģ̬Ӧģ̬


%% recurse on halves
if L > 1,
    
    nextlevel1 = mrDMD(x_dmd_lowrank,dt,max_cyc,L-1);
    nextlevel2 = mrDMD(x_dmd_sparse,dt,max_cyc,L-1);
else
    nextlleve1 = cell(0);
    nextlleve2 = cell(0);
end;

%% reconcile indexing on output
% (because Matlab does not support recursive data structures)
tree = cell(L, 2^(L-1));
tree{1,1} = thislevel;

for l = 2:L,
    col = 1;
    for j = 1:2^(l-2),
       tree{l, col} = nextlevel1{l-1, j};
       col = col + 1;
    end;
    for j = 1:2^(l-2)
       tree{l, col} = nextlevel2{l-1, j};
       col = col + 1;
    end;
end;